from chrome_spider.common.config import config
from chrome_spider.common.sql import DB
import json
import time

class CookieManager(object):
    def __init__(self, driver):
        self.driver = driver

    def save_cookie(self, host, name, cookies=None):
        """
        保存/更新 cookie 到数据库

        """
        if cookies == None:
            # 1. 由selenium 直接获取 cookie 保存 - 推荐使用
            cookies = self.driver.get_cookies()
        elif type(cookies) == str:
            # 2. 存储通过 document.cookies 获取的cookie - 有时候不能恢复登录态
            cookie_dic = dict([l.split("=", 1) for l in cookies.split("; ")])
            cookies = []
            for key, value in cookie_dic.items():
                cookies.append({"name": key, "value": value})

        cookies = json.dumps(cookies)

        with DB() as cursor:
            cursor.execute(
                "select cookie from account where host = ? and name = ?", (host, name)
            )
            if cursor.fetchone():
                print("update cookies")
                cursor.execute(
                    "update account set cookie = ? where name = ? and host = ?",
                    (cookies, name, host),
                )
            else:
                print("insert cookies")
                cursor.execute(
                    "insert into account (host,name,cookie) values (?,?,?)",
                    (host, name, cookies),
                )

    def login_by_cookie(self, host, name):
        """
        基于数据库存储的cookie进行登录
        @param host: 要登录的账号所在域名 示例: "weidian.com"
        @param name: 要登录的账号名称 示例: "kirito or 15233200000"
        """
        with DB() as cursor:
            cursor.execute(
                "select cookie from account where host = ? and name = ?", (host, name)
            )
            cookies = json.loads(cursor.fetchone()[0])
        for cookie in cookies:
            self.driver.add_cookie(cookie)
        time.sleep(0.5)
        self.driver.refresh()

    def get_cookie(self):
        """
        从数据库中获取 cookies
        """
        with DB() as cursor:
            cursor.execute(
                "select cookie from account where host = ? and name = ?", (host, name)
            )
            cookies = json.loads(cursor.fetchone()[0])
        return cookies



import os

from setuptools import setup,find_packages

here = os.path.abspath(os.path.dirname(__file__))

VERSION = "0.0.3"

with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="chrome-spider",
    version=VERSION,
    url="https://github.com/gaianote/chrome-spider",
    author="gaianote",
    author_email="gaianote311@gmail.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
    ],
    entry_points={
        "console_scripts": [
            "chrome-spider = chrome_spider.app:main"
        ]
    },
    install_requires=[
        "webdriver-manager >= 3.8.3",
        "selenium >= 4.4.3"
    ],
    extras_require={
        "dev": [
        ]
    },
)



from os import sep
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from webdriver_manager import chrome
from selenium import webdriver
import time
import json
import subprocess
from chrome_spider.common.sql import DB
from chrome_spider.common.config import config
from chrome_spider.spider.cookie import CookieManager



class Spider(CookieManager):
    def __init__(self,debug=False):
        self.debug = debug
        self.debugger_port = 9222


    def get_options(self):
        options = Options()
        options.add_argument("--ignore-certificate-errors")
        options.add_argument("--disable-features=EnableEphemeralFlashPermission")
        options.add_argument("--disable-blink-features=AutomationControlled")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        if self.debug == True:
            options.add_experimental_option("debuggerAddress", f"127.0.0.1:{self.debugger_port}")
        return options

    def open_debugger_port(self,debugger_port = 9222):
        shell = f'/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --remote-debugging-port={debugger_port} --user-data-dir="{config.chrome_data}"'
        subprocess.Popen(shell,shell=True)
        self.debugger_port = debugger_port
        time.sleep(3) # wait for chrome start

    def get_driver(self):

        self.driver = webdriver.Chrome(
            chrome.ChromeDriverManager().install(), options=self.get_options()
        )
        self.driver.set_page_load_timeout(30)
        self.driver.implicitly_wait(10)
        # self.driver.maximize_window()
        return self.driver

    




if __name__ == "__main__":
    # spider = ChromeSpider(debug=True)
    # spider.open_debugger_port()

    spider = ChromeSpider() 

    driver = spider.get_driver()
    driver.get("http://d.weidian.com")
    time.sleep(3)
    # spider.save_cookie("weidian.com","15233200179")
    
    spider.login_by_cookie("weidian.com","15233200179")
    driver.get("https://d.weidian.com/weidian-pc/weidian-loader/#/pc-vue-micro-index/index")
